@@ -21,18 +21,20 @@
 #include <malloc.h>
 #include <string.h>
 
+#include "afsd.h"
+
 #include <rx/rx.h>
 #include <rx/rx_prototypes.h>
+#include <WINNT/afsreg.h>
 
-#include "afsd.h"
 #include "afsicf.h"
 
 /* in seconds */
-long cm_daemonCheckDownInterval = 180;
-long cm_daemonCheckUpInterval = 600;
-long cm_daemonCheckVolInterval = 3600;
-long cm_daemonCheckCBInterval = 60;
-long cm_daemonCheckLockInterval = 60;
+long cm_daemonCheckDownInterval  = 180;
+long cm_daemonCheckUpInterval    = 600;
+long cm_daemonCheckVolInterval   = 3600;
+long cm_daemonCheckCBInterval    = 60;
+long cm_daemonCheckLockInterval  = 60;
 long cm_daemonTokenCheckInterval = 180;
 
 osi_rwlock_t cm_daemonLock;
@@ -75,7 +77,7 @@ void cm_BkgDaemon(long parm)
     lock_ObtainWrite(&cm_daemonLock);
     while (daemon_ShutdownFlag == 0) {
         if (!cm_bkgListEndp) {
-            osi_SleepW((long) &cm_bkgListp, &cm_daemonLock);
+            osi_SleepW((LONG_PTR)&cm_bkgListp, &cm_daemonLock);
             lock_ObtainWrite(&cm_daemonLock);
             continue;
         }
@@ -123,7 +125,7 @@ void cm_QueueBKGRequest(cm_scache_t *scp, cm_bkgProc_t *procp, long p1, long p2,
         cm_bkgListEndp = rp;
     lock_ReleaseWrite(&cm_daemonLock);
 
-    osi_Wakeup((long) &cm_bkgListp);
+    osi_Wakeup((LONG_PTR) &cm_bkgListp);
 }
 
 static int
@@ -142,13 +144,7 @@ IsWindowsFirewallPresent(void)
     if (!scm) return FALSE;
 
     /* Open Windows Firewall service */
-    svc = OpenService(scm, "MpsSvc", SERVICE_QUERY_CONFIG);
-    if (!svc) {
-	afsi_log("MpsSvc Service could not be opened for query: 0x%x", GetLastError());
-	svc = OpenService(scm, "SharedAccess", SERVICE_QUERY_CONFIG);
-	if (!svc)
-	    afsi_log("SharedAccess Service could not be opened for query: 0x%x", GetLastError());
-    }
+    svc = OpenService(scm, "SharedAccess", SERVICE_QUERY_CONFIG);
     if (!svc)
         goto close_scm;
 
@@ -166,13 +162,10 @@ IsWindowsFirewallPresent(void)
 
     /* Query Windows Firewall service config, this time for real */
     flag = QueryServiceConfig(svc, pConfig, BufSize, &BufSize);
-    if (!flag) {
-	afsi_log("QueryServiceConfig failed: 0x%x", GetLastError());
+    if (!flag)
         goto free_pConfig;
-    }
 
     /* Is it autostart? */
-    afsi_log("AutoStart 0x%x", pConfig->dwStartType);
     if (pConfig->dwStartType < SERVICE_DEMAND_START)
         result = TRUE;
 
@@ -186,26 +179,74 @@ IsWindowsFirewallPresent(void)
     return result;
 }
 
+void
+cm_DaemonCheckInit(void)
+{
+    HKEY parmKey;
+    DWORD dummyLen;
+    DWORD dummy;
+    DWORD code;
+
+    code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
+                         0, KEY_QUERY_VALUE, &parmKey);
+    if (code)
+	return;
+
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "DownServerCheckInterval", NULL, NULL,
+			    (BYTE *) &dummy, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckDownInterval = dummy;
+    
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "UpServerCheckInterval", NULL, NULL,
+			    (BYTE *) &dummy, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckUpInterval = dummy;
+    
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "VolumeCheckInterval", NULL, NULL,
+			    (BYTE *) &dummy, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckVolInterval = dummy;
+    
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "CallbackCheckInterval", NULL, NULL,
+			    (BYTE *) &dummy, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckCBInterval = dummy;
+    
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "LockCheckInterval", NULL, NULL,
+			    (BYTE *) &dummy, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonCheckLockInterval = dummy;
+    
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "TokenCheckInterval", NULL, NULL,
+			    (BYTE *) &dummy, &dummyLen);
+    if (code == ERROR_SUCCESS)
+	cm_daemonTokenCheckInterval = dummy;
+    
+    RegCloseKey(parmKey);
+}
+
 /* periodic check daemon */
 void cm_Daemon(long parm)
 {
-    unsigned long now;
-    unsigned long lastLockCheck;
-    unsigned long lastVolCheck;
-    unsigned long lastCBExpirationCheck;
-    unsigned long lastDownServerCheck;
-    unsigned long lastUpServerCheck;
-    unsigned long lastTokenCacheCheck;
+    time_t now;
+    time_t lastLockCheck;
+    time_t lastVolCheck;
+    time_t lastCBExpirationCheck;
+    time_t lastDownServerCheck;
+    time_t lastUpServerCheck;
+    time_t lastTokenCacheCheck;
     char thostName[200];
     unsigned long code;
     struct hostent *thp;
     HMODULE hHookDll;
     int configureFirewall = IsWindowsFirewallPresent();
 
-    if (!configureFirewall) {
-	afsi_log("No Windows Firewall detected");
-    }
-
     /* ping all file servers, up or down, with unauthenticated connection,
      * to find out whether we have all our callbacks from the server still.
      * Also, ping down VLDBs.
@@ -223,8 +264,11 @@ void cm_Daemon(long parm)
         code = 0;
     else
         memcpy(&code, thp->h_addr_list[0], 4);
+    
     srand(ntohl(code));
 
+    cm_DaemonCheckInit();
+
     now = osi_Time();
     lastVolCheck = now - cm_daemonCheckVolInterval/2 + (rand() % cm_daemonCheckVolInterval);
     lastCBExpirationCheck = now - cm_daemonCheckCBInterval/2 + (rand() % cm_daemonCheckCBInterval);
@@ -234,10 +278,6 @@ void cm_Daemon(long parm)
     lastTokenCacheCheck = now - cm_daemonTokenCheckInterval/2 + (rand() % cm_daemonTokenCheckInterval);
 
     while (daemon_ShutdownFlag == 0) {
-	thrd_Sleep(30 * 1000);		/* sleep 30 seconds */
-        if (daemon_ShutdownFlag == 1)
-            return;
-
 	if (configureFirewall) {
 	    /* Open Microsoft Firewall to allow in port 7001 */
 	    switch (icf_CheckAndAddAFSPorts(AFS_PORTSET_CLIENT)) {
@@ -257,7 +297,7 @@ void cm_Daemon(long parm)
 	    default:
 		afsi_log("Unknown Windows Firewall Configuration error");
 	    }
-	} 
+	}
 
         /* find out what time it is */
         now = osi_Time();
@@ -320,6 +360,10 @@ void cm_Daemon(long parm)
                 SetEvent(WaitToTerminate);
             }
         }
+
+	thrd_Sleep(30 * 1000);		/* sleep 30 seconds */
+        if (daemon_ShutdownFlag == 1)
+            return;
     }
 }       
 
